/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved

	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.

	This file is under Perforce control
	$Id: //depot/express/fcs992/modules/extr_scl.c#1 $
*/
#include <avs/util.h>
#include <avs/err.h>
#include <avs/om.h>
#include <avs/fld.h>
#include <avs/arr.h>
#include <avs/dv_util.h>

#define ERR_RETURN(A) ERRerror("extr_scalar",1,ERR_ORIG, A); return(0);
#define MAX_NAME_SIZE 1024
#define FIELDS_IN_SYNC(A,B) 0

int FUNCextr_scalar (OMobj_id in, int comp, OMobj_id out);

int DVextr_scalar_update(OMobj_id elem_id)
{
	OMobj_id in, out,comp_id;
	int comp;

	in = OMfind_subobj(elem_id, OMstr_to_name("in"), OM_OBJ_RD);
	out = OMfind_subobj(elem_id, OMstr_to_name("out"), OM_OBJ_RW);
	comp_id = OMfind_subobj(elem_id, OMstr_to_name("component"),
				     OM_OBJ_RW);

	OMget_int_val(comp_id, &comp);

	if (FUNCextr_scalar(in, comp, out)) {
		return(1);
	}
	else return(0);
}

int FUNCextr_scalar (OMobj_id in, int comp, OMobj_id out)
{
	int   nnodes, type, size, veclen, ncomp;
	int   data_type, null_flag, out_null_flag;
	char   *node_data, *out_node_data;
	double null_value;
	char  label[MAX_NAME_SIZE], tmp[32];

	if (FLDget_node_data_ncomp (in, &ncomp) != 1) {
		ERR_RETURN("cannot get node data ncomp");
	}
	if (ncomp < 1)
		return(0);

	if (FLDget_nnodes(in, &nnodes) != 1) {
		ERR_RETURN("cannot get nnodes");
	}

	if (FLDset_nnodes (out, nnodes) != 1) {
		ERR_RETURN("Error setting nnodes");
	}

	if (FLDget_node_data_veclen(in, 0, &veclen) != 1) {
		ERR_RETURN("Error getting veclen");
	}
	if (comp >= veclen) {
		if (FLDset_node_data_ncomp (out, 0) != 1) {
			ERR_RETURN("Error setting nnode_data");
		}
		ERR_RETURN("requested component does not exist, scalar is not created");
	}

	if (FLDset_node_data_ncomp (out, 1) != 1) {
		ERR_RETURN("Error setting nnode_data");
	}

	if (FLDget_node_data_label(in, 0, label, MAX_NAME_SIZE) != 1) {
		strcpy(label, "extract scalar");
	}
	else if (veclen > 1) {
		sprintf(tmp,"[%d]",comp);
		strcat(label, tmp);
	}

	if (FLDset_node_data_comp (out, 0, 1, label, "") != 1) {
		ERR_RETURN("Error setting node component");
	}

	if (FLDget_node_data(in, 0, &data_type, &node_data, 
			     &size, OM_GET_ARRAY_RD) != 1) {
		ERR_RETURN("cannot get node data");
	}
	if (FLDget_node_null_data(in, 0, &null_flag, (char *)&null_value) != 1) {
		ERR_RETURN("cannot get null data");
	}
	if (FLDget_node_data(out, 0, &data_type, &out_node_data, 
			     &size, OM_GET_ARRAY_WR) != 1) {
		ERR_RETURN("Error setting node data");
	}

	UTILextract_scalar(nnodes, veclen, node_data, data_type,
			   comp, out_node_data);
	if (null_flag) {
		if (FLDset_node_null_data(out, 0, (char *)&null_value, data_type) != 1) {
			ERR_RETURN("Error setting null value");
		}
	}
	else {
		if (FLDset_node_null_flag(out, 0, 0) != 1) {
			ERR_RETURN("Error setting null flag");
		}
	}
	ARRfree(node_data);
	ARRfree(out_node_data);
	return(1);
}
